package com.bolour.sample.eclipse.service.multiplication;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExecutableExtension;
import org.eclipse.ui.WorkbenchException;

import com.bolour.sample.eclipse.service.ui.IFunction;

/**
 * Callback class for the multiplication function. 
 * 
 * <p>
 * This class provides a generic multiplication function.
 * A particular instance of this class is parameterized by 
 * a specific factor to be used in multiplications. 
 * The factor is provided to this instance via a custom XML attribute called 
 * <i>constant</i> in its extension specification.
 * 
 * <p>
 * Because this class implements the <i>IExecutableExtension</i>
 * interface, the standard callback creation method of Eclipse,
 * namely, <code>IConfigurationElement.createExecutableExtension</code>
 * knows to initialize a created instance of the class by calling 
 * the <code>IExecutableExtension</code> method <code>setInitializationData</code>.
 * This method extracts the value of the <code>constant</code> attribute
 * and initializes the instance with its integer value as the factor
 * for later multiplications.
 * 
 * @author Azad
 */
public class Multiplication implements IFunction, IExecutableExtension {
	
	private static final String FACTOR_ATTRIBUTE = "constant";
	private int factor = 0;
		
	/**
	 * Implementation of the multiplication function.
	 */
	public long compute(long x) {		
		long result = x;
		result *= factor;
		return result;
	}

	/**
	 * Initialization method for instances of MultiplicationService.
	 * The multiplication factor for this particular listener is a custom XML attribute of its extension element.
	 * Extract the power and remember it for use in this operation.
	 * 
	 * @see org.eclipse.core.runtime.IExecutableExtension#setInitializationData(org.eclipse.core.runtime.IConfigurationElement, java.lang.String, java.lang.Object)
	 */
	public void setInitializationData(IConfigurationElement member, String propertyName, Object data) throws CoreException {
		
		String s = member.getAttribute(FACTOR_ATTRIBUTE);
		try {
			factor = Integer.parseInt(s);
		}
		catch (NumberFormatException ex) {
			System.err.println("Multiplication: unable to convert '" + FACTOR_ATTRIBUTE + "' attribute '" + s + "' to an integer.");
			throw new WorkbenchException("Multiplication: unable to convert '" + FACTOR_ATTRIBUTE + "' attribute '" + s + "' to an integer.", ex);
		}
	}
}
