package com.bolour.sample.eclipse.service.ui;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Device;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

/**
 * User interface grid for arithmetic functions.
 * Provides a function input and a function result field.
 * Allows user interface widgets (a button and an associated label) to be 
 * made available to the user for invoking a configured function.
 * 
 * @author Azad
 */
public class FunctionsGrid {
	private static final int LABEL_SIZE = 90;
	private static int ALIGN =
		GridData.HORIZONTAL_ALIGN_FILL | GridData.HORIZONTAL_ALIGN_CENTER;
	private Device device;
	private Color darkGreen = null;
	private Color red = null;
	private Color white = null;

	private Composite buttons;
	private Text input;
	private Label parameter;
	private Text result;

	/**
	 * Create the skeletal layout of the function invocation view.
	 * 
	 * @param parent The parent of the view.
	 */
	public FunctionsGrid(Composite parent) {
		// For good measure, get the device from parent to set up colors.
		device = parent.getShell().getDisplay();
		darkGreen = new Color(device, 60, 60, 30);
		red = new Color(device, 255, 0, 0);
		white = new Color(device, 255, 255, 255);

		// Tha main grid is used to lay out groups of components vertically.
		Composite main = new Composite(parent, SWT.NONE);
		GridLayout outerGrid = new GridLayout(1, false);
		outerGrid.horizontalSpacing = 5;
		outerGrid.verticalSpacing = 5;
		main.setLayout(outerGrid);

		// The fields grid is used to lay out the input and result fields.
		Composite fields = new Composite(main, SWT.NONE);
		fields.setLayoutData(new GridData(ALIGN));
		GridLayout fieldsLayout = new GridLayout(2, false);
		fieldsLayout.horizontalSpacing = 5;
		fieldsLayout.verticalSpacing = 5;
		fields.setLayout(fieldsLayout);

		// Lay out the fileds and their labels.

		// Input label.
		Label h = new Label(fields, SWT.NONE);
		h.setLayoutData(new GridData(ALIGN));
		h.setText("Input (number): ");

		// Input field.		
		input = new Text(fields, SWT.NONE);
		input.setLayoutData(new GridData(ALIGN));
		input.setForeground(darkGreen);
		input.setSize(300, 40);
		input.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				resetFields();
			}
		});
		
		// Result label.
		h = new Label(fields, SWT.NONE);
		h.setLayoutData(new GridData(ALIGN));
		h.setText("Result: ");

		// Result field.		
		result = new Text(fields, SWT.NONE);
		result.setEditable(false);
		result.setBackground(white);
		result.setLayoutData(new GridData(ALIGN));
		result.setSize(300, 40);

		// Constant label.
		h = new Label(fields, SWT.NONE);
		h.setLayoutData(new GridData(ALIGN));
		h.setText("Constant parameter: ");

		// Constant field.		
		parameter = new Label(fields, SWT.NONE);
		parameter.setLayoutData(new GridData(ALIGN));
		parameter.setText("none");
		parameter.setSize(300, 40);
		
		// The buttons grid is used to lay out function buttons and labels.
		buttons = new Composite(main, SWT.NONE);
		buttons.setLayoutData(new GridData(ALIGN));
		GridLayout layout = new GridLayout(2, false);
		layout.horizontalSpacing = 5;
		layout.verticalSpacing = 5;
		buttons.setLayout(layout);		
	}

	/**
	 * Add a new row to the functions grid for a member function.
	 * 
	 * @param function Reference to the function to call when 
	 * this row's button is selected.
	 * @param functionName The name of the function. 
	 * @param label Text of the associated label.
	 * @param constant The constant of computation (for display only).
	 */
	public void addFunction(IFunction function, String functionName, String label, Integer constant) {
		GridRow row = new GridRow(function, functionName, label, constant);
	}
	private void resetFields() {
		result.setText("");
		parameter.setText("");
	};

	/**
	 * A row of the <code>functions</code> grid. Represents a single function.
	 */
	private class GridRow {
		// Custom callback object for the function.
		private IFunction function;
		// Button to invoke the function.
		private Button button;
		// Label to display a fully-qualified ID for the service function.
		private Label functionLabel;
		// The constant of the computation, for display only.
		private String constantDisplay;

		/**
		 * Constructor: Create a new function row for the functions grid.
		 * 
		 * @param function The function associated with this row.
		 * @param functionName The display name of the function.
		 * @param label The text of the associated UI label.
		 * @param constant The constant of the calculation, if any (for display only).
		 */
		public GridRow(IFunction function, String functionName, String label, Integer constant) {
			GridData gridData = new GridData();
			gridData.horizontalAlignment = GridData.FILL;
			gridData.verticalAlignment = GridData.CENTER;
			gridData.grabExcessHorizontalSpace = true;
			gridData.grabExcessVerticalSpace = true;

			this.function = function;
			this.constantDisplay = constant == null ? "none" : constant.toString();

			button = new Button(buttons, SWT.NONE);
			button.setText(functionName);
			button.setLayoutData(new GridData(ALIGN));
			button.addSelectionListener(new SelectionListener() {
				public void widgetSelected(SelectionEvent e) {
					handleButton(e);
				}
				public void widgetDefaultSelected(SelectionEvent e) {
					widgetSelected(e);
				}
			});

			label = label.substring(0, Math.min(LABEL_SIZE, label.length()));
			functionLabel = new Label(buttons, SWT.NONE);
			functionLabel.setText(label);
			functionLabel.setLayoutData(new GridData(ALIGN));
			functionLabel.setForeground(red);
		}

		/**
		 * Event handler for the button click event. Invoke the function
		 * if its configured service and its input argument are valid.
		 * 
		 * @param e The function button event.
		 */
		public void handleButton(SelectionEvent e) {
			resetFields();
			String t = input.getText();
			try {
				int x = Integer.parseInt(t);
				result.setText(String.valueOf(function.compute(x)));
				parameter.setText(constantDisplay);
			}
			catch (NumberFormatException ex) {
				String message = "unable to convert input '" + t + "' to integer";
				System.err.println(message);
				result.setText("error");
			}
			catch (ArithmeticException ex) {
				System.err.println(ex.getMessage());
				result.setText("error");
			}
		}
	}
}
